#!/usr/bin/env python
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import datetime as dtm

import pytest

from telegram import (
    User,
    VideoChatEnded,
    VideoChatParticipantsInvited,
    VideoChatScheduled,
    VideoChatStarted,
)
from telegram._utils.datetime import UTC, to_timestamp
from telegram.warnings import PTBDeprecationWarning
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def user1():
    return User(first_name="Misses Test", id=123, is_bot=False)


@pytest.fixture(scope="module")
def user2():
    return User(first_name="Mister Test", id=124, is_bot=False)


class TestVideoChatStartedWithoutRequest:
    def test_slot_behaviour(self):
        action = VideoChatStarted()
        for attr in action.__slots__:
            assert getattr(action, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(action)) == len(set(mro_slots(action))), "duplicate slot"

    def test_de_json(self):
        video_chat_started = VideoChatStarted.de_json({}, None)
        assert video_chat_started.api_kwargs == {}
        assert isinstance(video_chat_started, VideoChatStarted)

    def test_to_dict(self):
        video_chat_started = VideoChatStarted()
        video_chat_dict = video_chat_started.to_dict()
        assert video_chat_dict == {}


class TestVideoChatEndedWithoutRequest:
    duration = dtm.timedelta(seconds=100)

    def test_slot_behaviour(self):
        action = VideoChatEnded(8)
        for attr in action.__slots__:
            assert getattr(action, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(action)) == len(set(mro_slots(action))), "duplicate slot"

    def test_de_json(self):
        json_dict = {"duration": int(self.duration.total_seconds())}
        video_chat_ended = VideoChatEnded.de_json(json_dict, None)
        assert video_chat_ended.api_kwargs == {}

        assert video_chat_ended._duration == self.duration

    def test_to_dict(self):
        video_chat_ended = VideoChatEnded(self.duration)
        video_chat_dict = video_chat_ended.to_dict()

        assert isinstance(video_chat_dict, dict)
        assert video_chat_dict["duration"] == int(self.duration.total_seconds())

    def test_time_period_properties(self, PTB_TIMEDELTA):
        duration = VideoChatEnded(duration=self.duration).duration

        if PTB_TIMEDELTA:
            assert duration == self.duration
            assert isinstance(duration, dtm.timedelta)
        else:
            assert duration == int(self.duration.total_seconds())
            assert isinstance(duration, int)

    def test_time_period_int_deprecated(self, recwarn, PTB_TIMEDELTA):
        VideoChatEnded(self.duration).duration

        if PTB_TIMEDELTA:
            assert len(recwarn) == 0
        else:
            assert len(recwarn) == 1
            assert "`duration` will be of type `datetime.timedelta`" in str(recwarn[0].message)
            assert recwarn[0].category is PTBDeprecationWarning

    def test_equality(self):
        a = VideoChatEnded(100)
        b = VideoChatEnded(100)
        x = VideoChatEnded(dtm.timedelta(seconds=100))
        c = VideoChatEnded(50)
        d = VideoChatStarted()

        assert a == b
        assert hash(a) == hash(b)
        assert b == x
        assert hash(b) == hash(x)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


class TestVideoChatParticipantsInvitedWithoutRequest:
    def test_slot_behaviour(self, user1):
        action = VideoChatParticipantsInvited([user1])
        for attr in action.__slots__:
            assert getattr(action, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(action)) == len(set(mro_slots(action))), "duplicate slot"

    def test_de_json(self, user1, user2, offline_bot):
        json_data = {"users": [user1.to_dict(), user2.to_dict()]}
        video_chat_participants = VideoChatParticipantsInvited.de_json(json_data, offline_bot)
        assert video_chat_participants.api_kwargs == {}

        assert isinstance(video_chat_participants.users, tuple)
        assert video_chat_participants.users[0] == user1
        assert video_chat_participants.users[1] == user2
        assert video_chat_participants.users[0].id == user1.id
        assert video_chat_participants.users[1].id == user2.id

    @pytest.mark.parametrize("use_users", [True, False])
    def test_to_dict(self, user1, user2, use_users):
        video_chat_participants = VideoChatParticipantsInvited([user1, user2] if use_users else ())
        video_chat_dict = video_chat_participants.to_dict()

        assert isinstance(video_chat_dict, dict)
        if use_users:
            assert video_chat_dict["users"] == [user1.to_dict(), user2.to_dict()]
            assert video_chat_dict["users"][0]["id"] == user1.id
            assert video_chat_dict["users"][1]["id"] == user2.id
        else:
            assert video_chat_dict == {}

    def test_equality(self, user1, user2):
        a = VideoChatParticipantsInvited([user1])
        b = VideoChatParticipantsInvited([user1])
        c = VideoChatParticipantsInvited([user1, user2])
        d = VideoChatParticipantsInvited([])
        e = VideoChatStarted()

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)


class TestVideoChatScheduledWithoutRequest:
    start_date = dtm.datetime.now(dtm.timezone.utc)

    def test_slot_behaviour(self):
        inst = VideoChatScheduled(self.start_date)
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self):
        assert VideoChatScheduled(self.start_date).start_date == self.start_date

    def test_de_json(self, offline_bot):
        json_dict = {"start_date": to_timestamp(self.start_date)}
        video_chat_scheduled = VideoChatScheduled.de_json(json_dict, offline_bot)
        assert video_chat_scheduled.api_kwargs == {}

        assert abs(video_chat_scheduled.start_date - self.start_date) < dtm.timedelta(seconds=1)

    def test_de_json_localization(self, tz_bot, offline_bot, raw_bot):
        json_dict = {"start_date": to_timestamp(self.start_date)}

        videochat_raw = VideoChatScheduled.de_json(json_dict, raw_bot)
        videochat_bot = VideoChatScheduled.de_json(json_dict, offline_bot)
        videochat_tz = VideoChatScheduled.de_json(json_dict, tz_bot)

        # comparing utcoffsets because comparing timezones is unpredicatable
        videochat_offset = videochat_tz.start_date.utcoffset()
        tz_bot_offset = tz_bot.defaults.tzinfo.utcoffset(
            videochat_tz.start_date.replace(tzinfo=None)
        )

        assert videochat_raw.start_date.tzinfo == UTC
        assert videochat_bot.start_date.tzinfo == UTC
        assert videochat_offset == tz_bot_offset

    def test_to_dict(self):
        video_chat_scheduled = VideoChatScheduled(self.start_date)
        video_chat_scheduled_dict = video_chat_scheduled.to_dict()

        assert isinstance(video_chat_scheduled_dict, dict)
        assert video_chat_scheduled_dict["start_date"] == to_timestamp(self.start_date)

    def test_equality(self):
        a = VideoChatScheduled(self.start_date)
        b = VideoChatScheduled(self.start_date)
        c = VideoChatScheduled(dtm.datetime.utcnow() + dtm.timedelta(seconds=5))
        d = VideoChatStarted()

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)



#
#  A library that provides a Python interface to the Telegram Bot API
#  Copyright (C) 2015-2026
#  Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser Public License for more details.
#
#  You should have received a copy of the GNU Lesser Public License
#  along with this program.  If not, see [http://www.gnu.org/licenses/].
"""Functionality in this file is used for getting the [source] links on the classes, methods etc
to link to the correct files & lines on github. Can be simplified once
https://github.com/sphinx-doc/sphinx/issues/1556 is closed
"""

import subprocess
from pathlib import Path

from sphinx.util import logging

# get the sphinx(!) logger
# Makes sure logs render in red and also plays nicely with e.g. the `nitpicky` option.
sphinx_logger = logging.getLogger(__name__)


# must be a module-level variable so that it can be written to by the `autodoc-process-docstring`
# event handler in `sphinx_hooks.py`
LINE_NUMBERS: dict[str, tuple[Path, int, int]] = {}


def _git_branch() -> str:
    """Get's the current git sha if available or fall back to `master`"""
    try:
        output = subprocess.check_output(
            ["git", "describe", "--tags", "--always"], stderr=subprocess.STDOUT
        )
        return output.decode().strip()
    except Exception as exc:
        sphinx_logger.exception(
            "Failed to get a description of the current commit. Falling back to `master`.",
            exc_info=exc,
        )
        return "master"


git_branch = _git_branch()
base_url = "https://github.com/python-telegram-bot/python-telegram-bot/blob/"


def linkcode_resolve(_, info) -> str:
    """See www.sphinx-doc.org/en/master/usage/extensions/linkcode.html"""
    combined = ".".join((info["module"], info["fullname"]))
    # special casing for ExtBot which is due to the special structure of extbot.rst
    combined = combined.replace("ExtBot.ExtBot", "ExtBot")

    line_info = LINE_NUMBERS.get(combined)

    if not line_info:
        # Try the __init__
        line_info = LINE_NUMBERS.get(f"{combined.rsplit('.', 1)[0]}.__init__")
    if not line_info:
        # Try the class
        line_info = LINE_NUMBERS.get(f"{combined.rsplit('.', 1)[0]}")
    if not line_info:
        # Try the module
        line_info = LINE_NUMBERS.get(info["module"])

    if not line_info:
        return None

    file, start_line, end_line = line_info
    return f"{base_url}{git_branch}/{file}#L{start_line}-L{end_line}"



#!/usr/bin/env python
# pylint: disable=unused-argument
# This program is dedicated to the public domain under the CC0 license.

"""
Simple Bot to reply to Telegram messages.

First, a few handler functions are defined. Then, those functions are passed to
the Application and registered at their respective places.
Then, the bot is started and runs until we press Ctrl-C on the command line.

Usage:
Basic Echobot example, repeats messages.
Press Ctrl-C on the command line or send a signal to the process to stop the
bot.
"""

import logging

from telegram import ForceReply, Update
from telegram.ext import Application, CommandHandler, ContextTypes, MessageHandler, filters

# Enable logging
logging.basicConfig(
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s", level=logging.INFO
)
# set higher logging level for httpx to avoid all GET and POST requests being logged
logging.getLogger("httpx").setLevel(logging.WARNING)

logger = logging.getLogger(__name__)


# Define a few command handlers. These usually take the two arguments update and
# context.
async def start(update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
    """Send a message when the command /start is issued."""
    user = update.effective_user
    await update.message.reply_html(
        rf"Hi {user.mention_html()}!",
        reply_markup=ForceReply(selective=True),
    )


async def help_command(update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
    """Send a message when the command /help is issued."""
    await update.message.reply_text("Help!")


async def echo(update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
    """Echo the user message."""
    await update.message.reply_text(update.message.text)


def main() -> None:
    """Start the bot."""
    # Create the Application and pass it your bot's token.
    application = Application.builder().token("TOKEN").build()

    # on different commands - answer in Telegram
    application.add_handler(CommandHandler("start", start))
    application.add_handler(CommandHandler("help", help_command))

    # on non command i.e message - echo the message on Telegram
    application.add_handler(MessageHandler(filters.TEXT & ~filters.COMMAND, echo))

    # Run the bot until the user presses Ctrl-C
    application.run_polling(allowed_updates=Update.ALL_TYPES)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import pytest

from telegram import (
    __bot_api_version__,
    __bot_api_version_info__,
    __version__,
    __version_info__,
    constants,
)
from telegram._version import Version


class TestVersion:
    def test_bot_api_version_and_info(self):
        assert __bot_api_version__ is constants.BOT_API_VERSION
        assert __bot_api_version_info__ is constants.BOT_API_VERSION_INFO

    def test_version_and_info(self):
        assert __version__ == str(__version_info__)

    @pytest.mark.parametrize(
        ("version", "expected"),
        [
            (Version(1, 2, 3, "alpha", 4), "1.2.3a4"),
            (Version(2, 3, 4, "beta", 5), "2.3.4b5"),
            (Version(1, 2, 3, "candidate", 4), "1.2.3rc4"),
            (Version(1, 2, 0, "alpha", 4), "1.2a4"),
            (Version(2, 3, 0, "beta", 5), "2.3b5"),
            (Version(1, 2, 0, "candidate", 4), "1.2rc4"),
            (Version(1, 2, 3, "final", 0), "1.2.3"),
            (Version(1, 2, 0, "final", 0), "1.2"),
        ],
    )
    def test_version_str(self, version, expected):
        assert str(version) == expected

    @pytest.mark.parametrize("use_tuple", [True, False])
    def test_version_info(self, use_tuple):
        version = Version(1, 2, 3, "beta", 4)
        assert isinstance(version, tuple)
        assert version.major == version[0]
        assert version.minor == version[1]
        assert version.micro == version[2]
        assert version.releaselevel == version[3]
        assert version.serial == version[4]

        class TestClass:
            def __new__(cls, *args):
                if use_tuple:
                    return tuple(args)
                return Version(*args)

        assert isinstance(TestClass(1, 2, 3, "beta", 4), tuple if use_tuple else Version)
        assert version == TestClass(1, 2, 3, "beta", 4)
        assert not (version < TestClass(1, 2, 3, "beta", 4))
        assert version > TestClass(1, 2, 3, "beta", 3)
        assert version > TestClass(1, 2, 3, "alpha", 4)
        assert version < TestClass(1, 2, 3, "candidate", 0)
        assert version < TestClass(1, 2, 3, "final", 0)
        assert version < TestClass(1, 2, 4, "final", 0)
        assert version < TestClass(1, 3, 4, "final", 0)

        assert version < (1, 3)
        assert version >= (1, 2, 3, "alpha")
        assert version > (1, 1)
        assert version <= (1, 2, 3, "beta", 4)
        assert version < (1, 2, 3, "candidate", 4)
        assert not (version > (1, 2, 3, "candidate", 4))
        assert version < (1, 2, 4)
        assert version > (1, 2, 2)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains convenience helper functions.

.. versionchanged:: 20.0
   Previously, the contents of this module were available through the (no longer existing)
   module ``telegram.utils.helpers``.
"""

__all__ = (
    "create_deep_linked_url",
    "effective_message_type",
    "escape_markdown",
    "mention_html",
    "mention_markdown",
)

import re
from html import escape
from typing import TYPE_CHECKING

from telegram._utils.types import MarkdownVersion
from telegram.constants import MessageLimit, MessageType

if TYPE_CHECKING:
    from telegram import Message, Update


def escape_markdown(
    text: str, version: MarkdownVersion = 1, entity_type: str | None = None
) -> str:
    """Helper function to escape telegram markup symbols.

    .. versionchanged:: 20.3
        Custom emoji entity escaping is now supported.

    Args:
        text (:obj:`str`): The text.
        version (:obj:`int` | :obj:`str`): Use to specify the version of telegrams Markdown.
            Either ``1`` or ``2``. Defaults to ``1``.
        entity_type (:obj:`str`, optional): For the entity types
            :tg-const:`telegram.MessageEntity.PRE`, :tg-const:`telegram.MessageEntity.CODE` and
            the link part of :tg-const:`telegram.MessageEntity.TEXT_LINK` and
            :tg-const:`telegram.MessageEntity.CUSTOM_EMOJI`, only certain characters need to be
            escaped in :tg-const:`telegram.constants.ParseMode.MARKDOWN_V2`. See the `official API
            documentation <https://core.telegram.org/bots/api#formatting-options>`_ for details.
            Only valid in combination with ``version=2``, will be ignored else.
    """
    if int(version) == 1:
        escape_chars = r"_*`["
    elif int(version) == 2:
        if entity_type in ["pre", "code"]:
            escape_chars = r"\`"
        elif entity_type in ["text_link", "custom_emoji"]:
            escape_chars = r"\)"
        else:
            escape_chars = r"\_*[]()~`>#+-=|{}.!"
    else:
        raise ValueError("Markdown version must be either 1 or 2!")

    return re.sub(f"([{re.escape(escape_chars)}])", r"\\\1", text)


def mention_html(user_id: int | str, name: str) -> str:
    """
    Helper function to create a user mention as HTML tag.

    Args:
        user_id (:obj:`int`): The user's id which you want to mention.
        name (:obj:`str`): The name the mention is showing.

    Returns:
        :obj:`str`: The inline mention for the user as HTML.
    """
    return f'<a href="tg://user?id={user_id}">{escape(name)}</a>'


def mention_markdown(user_id: int | str, name: str, version: MarkdownVersion = 1) -> str:
    """
    Helper function to create a user mention in Markdown syntax.

    Args:
        user_id (:obj:`int`): The user's id which you want to mention.
        name (:obj:`str`): The name the mention is showing.
        version (:obj:`int` | :obj:`str`): Use to specify the version of Telegram's Markdown.
            Either ``1`` or ``2``. Defaults to ``1``.

    Returns:
        :obj:`str`: The inline mention for the user as Markdown.
    """
    tg_link = f"tg://user?id={user_id}"
    if version == 1:
        return f"[{name}]({tg_link})"
    return f"[{escape_markdown(name, version=version)}]({tg_link})"


def effective_message_type(entity: "Message | Update") -> str | None:
    """
    Extracts the type of message as a string identifier from a :class:`telegram.Message` or a
    :class:`telegram.Update`.

    Args:
        entity (:class:`telegram.Update` | :class:`telegram.Message`): The ``update`` or
            ``message`` to extract from.

    Returns:
        :obj:`str` | :obj:`None`: One of :class:`telegram.constants.MessageType` if the entity
        contains a message that matches one of those types. :obj:`None` otherwise.

    """
    # Importing on file-level yields cyclic Import Errors
    from telegram import (  # pylint: disable=import-outside-toplevel  # noqa: PLC0415
        Message,
        Update,
    )

    if isinstance(entity, Message):
        message = entity
    elif isinstance(entity, Update):
        if not entity.effective_message:
            return None
        message = entity.effective_message
    else:
        raise TypeError(f"The entity is neither Message nor Update (got: {type(entity)})")

    for message_type in MessageType:
        if message[message_type]:
            return message_type

    return None


def create_deep_linked_url(
    bot_username: str, payload: str | None = None, group: bool = False
) -> str:
    """
    Creates a deep-linked URL for this :paramref:`~create_deep_linked_url.bot_username` with the
    specified :paramref:`~create_deep_linked_url.payload`. See
    https://core.telegram.org/bots/features#deep-linking to learn more.

    The :paramref:`~create_deep_linked_url.payload` may consist of the following characters:
    ``A-Z, a-z, 0-9, _, -``

    Note:
        Works well in conjunction with
        ``CommandHandler("start", callback, filters=filters.Regex('payload'))``

    Examples:
        * ``create_deep_linked_url(bot.get_me().username, "some-params")``
        * :any:`Deep Linking <examples.deeplinking>`

    Args:
        bot_username (:obj:`str`): The username to link to.
        payload (:obj:`str`, optional): Parameters to encode in the created URL.
        group (:obj:`bool`, optional): If :obj:`True` the user is prompted to select a group to
            add the bot to. If :obj:`False`, opens a one-on-one conversation with the bot.
            Defaults to :obj:`False`.

    Returns:
        :obj:`str`: An URL to start the bot with specific parameters.

    Raises:
        :exc:`ValueError`: If the length of the :paramref:`payload` exceeds \
        :tg-const:`telegram.constants.MessageLimit.DEEP_LINK_LENGTH` characters,
            contains invalid characters, or if the :paramref:`bot_username` is less than 4
            characters.
    """
    if bot_username is None or len(bot_username) <= 3:
        raise ValueError("You must provide a valid bot_username.")

    base_url = f"https://t.me/{bot_username}"
    if not payload:
        return base_url

    if len(payload) > MessageLimit.DEEP_LINK_LENGTH:
        raise ValueError(
            f"The deep-linking payload must not exceed {MessageLimit.DEEP_LINK_LENGTH} characters."
        )

    if not re.match(r"^[A-Za-z0-9_-]+$", payload):
        raise ValueError(
            "Only the following characters are allowed for deep-linked "
            "URLs: A-Z, a-z, 0-9, _ and -"
        )

    key = "startgroup" if group else "start"

    return f"{base_url}?{key}={payload}"



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import pytest

from telegram import Bot, OrderInfo, PreCheckoutQuery, Update, User
from tests.auxil.bot_method_checks import (
    check_defaults_handling,
    check_shortcut_call,
    check_shortcut_signature,
)
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def pre_checkout_query(bot):
    pcq = PreCheckoutQuery(
        PreCheckoutQueryTestBase.id_,
        PreCheckoutQueryTestBase.from_user,
        PreCheckoutQueryTestBase.currency,
        PreCheckoutQueryTestBase.total_amount,
        PreCheckoutQueryTestBase.invoice_payload,
        shipping_option_id=PreCheckoutQueryTestBase.shipping_option_id,
        order_info=PreCheckoutQueryTestBase.order_info,
    )
    pcq.set_bot(bot)
    return pcq


class PreCheckoutQueryTestBase:
    id_ = 5
    invoice_payload = "invoice_payload"
    shipping_option_id = "shipping_option_id"
    currency = "EUR"
    total_amount = 100
    from_user = User(0, "", False)
    order_info = OrderInfo()


class TestPreCheckoutQueryWithoutRequest(PreCheckoutQueryTestBase):
    def test_slot_behaviour(self, pre_checkout_query):
        inst = pre_checkout_query
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "id": self.id_,
            "invoice_payload": self.invoice_payload,
            "shipping_option_id": self.shipping_option_id,
            "currency": self.currency,
            "total_amount": self.total_amount,
            "from": self.from_user.to_dict(),
            "order_info": self.order_info.to_dict(),
        }
        pre_checkout_query = PreCheckoutQuery.de_json(json_dict, offline_bot)
        assert pre_checkout_query.api_kwargs == {}

        assert pre_checkout_query.get_bot() is offline_bot
        assert pre_checkout_query.id == self.id_
        assert pre_checkout_query.invoice_payload == self.invoice_payload
        assert pre_checkout_query.shipping_option_id == self.shipping_option_id
        assert pre_checkout_query.currency == self.currency
        assert pre_checkout_query.from_user == self.from_user
        assert pre_checkout_query.order_info == self.order_info

    def test_to_dict(self, pre_checkout_query):
        pre_checkout_query_dict = pre_checkout_query.to_dict()

        assert isinstance(pre_checkout_query_dict, dict)
        assert pre_checkout_query_dict["id"] == pre_checkout_query.id
        assert pre_checkout_query_dict["invoice_payload"] == pre_checkout_query.invoice_payload
        assert (
            pre_checkout_query_dict["shipping_option_id"] == pre_checkout_query.shipping_option_id
        )
        assert pre_checkout_query_dict["currency"] == pre_checkout_query.currency
        assert pre_checkout_query_dict["from"] == pre_checkout_query.from_user.to_dict()
        assert pre_checkout_query_dict["order_info"] == pre_checkout_query.order_info.to_dict()

    def test_equality(self):
        a = PreCheckoutQuery(
            self.id_, self.from_user, self.currency, self.total_amount, self.invoice_payload
        )
        b = PreCheckoutQuery(
            self.id_, self.from_user, self.currency, self.total_amount, self.invoice_payload
        )
        c = PreCheckoutQuery(self.id_, None, "", 0, "")
        d = PreCheckoutQuery(
            0, self.from_user, self.currency, self.total_amount, self.invoice_payload
        )
        e = Update(self.id_)

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

    async def test_answer(self, monkeypatch, pre_checkout_query):
        async def make_assertion(*_, **kwargs):
            return kwargs["pre_checkout_query_id"] == pre_checkout_query.id

        assert check_shortcut_signature(
            PreCheckoutQuery.answer, Bot.answer_pre_checkout_query, ["pre_checkout_query_id"], []
        )
        assert await check_shortcut_call(
            pre_checkout_query.answer,
            pre_checkout_query.get_bot(),
            "answer_pre_checkout_query",
        )
        assert await check_defaults_handling(
            pre_checkout_query.answer, pre_checkout_query.get_bot()
        )

        monkeypatch.setattr(
            pre_checkout_query.get_bot(), "answer_pre_checkout_query", make_assertion
        )
        assert await pre_checkout_query.answer(ok=True)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import pytest

from telegram import LinkPreviewOptions
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def link_preview_options():
    return LinkPreviewOptions(
        is_disabled=LinkPreviewOptionsTestBase.is_disabled,
        url=LinkPreviewOptionsTestBase.url,
        prefer_small_media=LinkPreviewOptionsTestBase.prefer_small_media,
        prefer_large_media=LinkPreviewOptionsTestBase.prefer_large_media,
        show_above_text=LinkPreviewOptionsTestBase.show_above_text,
    )


class LinkPreviewOptionsTestBase:
    is_disabled = True
    url = "https://www.example.com"
    prefer_small_media = True
    prefer_large_media = False
    show_above_text = True


class TestLinkPreviewOptionsWithoutRequest(LinkPreviewOptionsTestBase):
    def test_slot_behaviour(self, link_preview_options):
        a = link_preview_options
        for attr in a.__slots__:
            assert getattr(a, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(a)) == len(set(mro_slots(a))), "duplicate slot"

    def test_to_dict(self, link_preview_options):
        link_preview_options_dict = link_preview_options.to_dict()

        assert isinstance(link_preview_options_dict, dict)
        assert link_preview_options_dict["is_disabled"] == self.is_disabled
        assert link_preview_options_dict["url"] == self.url
        assert link_preview_options_dict["prefer_small_media"] == self.prefer_small_media
        assert link_preview_options_dict["prefer_large_media"] == self.prefer_large_media
        assert link_preview_options_dict["show_above_text"] == self.show_above_text

    def test_de_json(self, link_preview_options):
        link_preview_options_dict = {
            "is_disabled": self.is_disabled,
            "url": self.url,
            "prefer_small_media": self.prefer_small_media,
            "prefer_large_media": self.prefer_large_media,
            "show_above_text": self.show_above_text,
        }

        link_preview_options = LinkPreviewOptions.de_json(link_preview_options_dict, bot=None)
        assert link_preview_options.api_kwargs == {}

        assert link_preview_options.is_disabled == self.is_disabled
        assert link_preview_options.url == self.url
        assert link_preview_options.prefer_small_media == self.prefer_small_media
        assert link_preview_options.prefer_large_media == self.prefer_large_media
        assert link_preview_options.show_above_text == self.show_above_text

    def test_equality(self):
        a = LinkPreviewOptions(
            self.is_disabled,
            self.url,
            self.prefer_small_media,
            self.prefer_large_media,
            self.show_above_text,
        )
        b = LinkPreviewOptions(
            self.is_disabled,
            self.url,
            self.prefer_small_media,
            self.prefer_large_media,
            self.show_above_text,
        )
        c = LinkPreviewOptions(self.is_disabled)
        d = LinkPreviewOptions(
            False, self.url, self.prefer_small_media, self.prefer_large_media, self.show_above_text
        )

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)
